clc;
clear all;
% print debug information
debug = 1;
% length of input signal
lengthX = 540;
% generate random input signal
x = randn(1,lengthX)+2;
% load defined schemes
load('schemes.mat');
% choose one
ls = ls7_9;
% print scheme
ls

% levels 1:5
for level = 1:5
    disp('   ');
    disp('%%%%%%% Performing Segmented Lifting Wavelet Transform %%%%%%%%%%%%%');
    disp(sprintf('Signal length: %d. Level: %d',lengthX,level));
    disp('Two segments.');
    

% all possible cases of division in actual level (extensiona are periodic with 2^J)
for it = 0:-1:-2^level-1

%%%%% LWT of whole signal %%%  
[Xtemp] = mylwt(x,ls,level);
[x_hat_orig] = myilwt(Xtemp,ls,level);
Xlift = lift2cell(Xtemp,level);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% decide where to split %%%
xHalf = ceil(length(x)/2) +it;
lastIndex = xHalf-1;
firstIndex = lastIndex +1;
%%%%%%%%%%%%%%%%%%%%%%%%

% calculate extensions
[left,right,lefts,rights]=extJ(ls,firstIndex,level);


%%% OPTIONALLY trade shift*2^(level) between left and right extensions
shift = 0;
extMove = shift* 2^(level);
rightExt = right + extMove;
leftExt = left - extMove;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% SOME DEBUG information %%%%%%
 if debug ~= 0   
  disp('   ');
  disp(['lastInd=',num2str(lastIndex),...
        ' firstInd=',num2str(firstIndex),...
      ' leftExtension= ', num2str(leftExt), ' rightExtension= ', num2str(rightExt), ' sum= ', num2str(leftExt+rightExt) ]);

  for i =1:length(lefts)
       disp(sprintf('Extension in level %d: left %d, right %d, sum=%d',i,lefts(i),rights(i),lefts(i)+rights(i)));
  end
 end
  
 if xHalf+rightExt > lengthX
     disp(sprintf('Right extension overflows. Signal length is %d, first segment with right extension is %d. Skipping...',lengthX,xHalf+rightExt));
     continue;
 end;
 
 if xHalf+1-leftExt < 1
     disp(sprintf('Left extension underflows by %d. Skipping...',xHalf+1-leftExt));
     continue;
 end;
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
%%%% Creating two segments with extensions 
x1ext = x(1:xHalf+rightExt);
x2ext = x(xHalf+1-leftExt:end);
%%%%%%%%%%%%

%%%%%% LWT of first segment %%%%%%
[A1] = mylwt(x1ext,ls,level);
Xlift1temp = lift2cell(A1,level);

%%%%%% LWT of second segment %%%%%%%
[A2] = mylwt(x2ext,ls,level,firstIndex-leftExt);
Xlift2temp = lift2cell(A2,level,firstIndex-leftExt);

%%%%%% Crop redundat coefficients %%%%%%%%%
Xlift1 = cropRight(Xlift1temp,right,lastIndex );
Xlift2 = cropLeft(Xlift2temp,left,firstIndex );

%%% Check transforms %%%
MSE = checkTransform(Xlift,[Xlift1,Xlift2]);
if debug ~= 0  disp(['MSE ve wavelet. oblasti= ', num2str(MSE)]); end

%%%%%% Calculate extension for inverse transform
[left,right,lefts,rights]=extJinv(ls,firstIndex,level);

%%%%% Additional debug information %%%%%%
 if debug ~= 0  
  disp('Inverse transform:')   
  disp(['leftExtension= ', num2str(right), ' rightExtension= ', num2str(left), ' sum= ', num2str(left+right) ]);
  for i =1:length(lefts)
       disp(sprintf('Extension in level %d: left %d, right %d, sum=%d',i,rights(i),lefts(i),lefts(i)+rights(i)));
  end
 end

%%%%%%% Inverse transform of first segment
inverseExt = left; % ! not used right
Xlift1temp = cell2lift(Xlift1,level)';
Xlift1temp = [Xlift1temp zeros(1,inverseExt)];
X1_hat = myilwt(Xlift1temp,ls,level);

%%%%%%% Inverse transform of second segment
inverseExt = right; % !! not used left
Xlift2temp = cell2lift(Xlift2,level,firstIndex)';
Xlift2temp = [zeros(1,inverseExt), Xlift2temp];
X2_hat = myilwt(Xlift2temp,ls,level,firstIndex-inverseExt);


%% plot results
plots = cell(level+1,1);

for j=1:level
    plots{j} = zeros(length(Xlift{j}),3);
    plots{j}(:,1) = Xlift{j}';
    plots{j}(1:length(Xlift1{j}),2) = Xlift1{j}';
    plots{j}(end-length(Xlift2{j})+1:end,3) = Xlift2{j}';
end

plots{level+1} = zeros(length(Xlift{level+1}),3);
plots{level+1}(:,1) = Xlift{level+1}';
plots{level+1}(1:length(Xlift1{level+1}),2) = Xlift1{level+1}';
plots{level+1}(end-length(Xlift2{level+1})+1:end,3) = Xlift2{level+1}';

  first = [X1_hat, zeros(1,length(x)-length(X1_hat)) ]';
  second = [zeros(1,length(x)-length(X2_hat)), X2_hat  ]';
  suma = first + second;
 
  %%% check reconstruction %%%
  MSE = sum(abs(x_hat_orig'-suma));
  if(MSE>10^-8)  disp(['MSE after reconstruction= ', num2str(MSE)]); end;

if debug ~= 0 
   %% figure 1 shows wavelet coefficients in all levels  
figure(1); 
clf;
for j = 1:level+1
    subplot(level+1,1,j);
    stem(plots{j}(:,1));
end
pause;
clf;
for j = 1:level+1
    subplot(level+1,1,j);
    stem(plots{j}(:,1:2));
end
pause;
clf;
for j = 1:level+1
    subplot(level+1,1,j);
    stem(plots{j}(:,[1,3]));
end
 pause;
 %%% figure 2 shows original and reconstructed signals
  figure(2);
 clf;
  subplot(2,1,1);
osax = 0:length(x')-1;
    stem(osax,[x_hat_orig',suma]);
    subplot(2,1,2);
    stem(osax,[first,second]);
 end    
    
   

  
 
 end;
end;
 


    
 


