function [leftExtension,rightExtension] = extensions( ls, S )
%%EXTENSIONS One level left and right extensions for SLWT
%   [L,R] = extensions( LS, S ) returns scalar values L,R for one level
%   extensions computed from lifting scheme LS and index S(1) as a
%   beginning of the segment and S(1)-1 or S(2) as end of segment.
%
%   Implements Algorithm 1 from the paper Z. Prusa and P. Rajmic: Segmented
%   computation of wavelet transform via lifting scheme, 2011
%
%   See also: extJ

%   Author(s): Z. Prusa
%   $Revision: 1.0 $  $Date: 2011/05/02$ 


[K,L]=size(ls);
beginS = S(1);
if(length(S>1))
   endS = S(2);
else
   endS = beginS - 1;
end

leftExtension = 0;
rightExtension = 0;
parBegin = mod(beginS,2);
parEnd = mod(endS,2);

for k=K-1:-1:1
    
    tempLeftExtension = 0;
    tempRightExtension = 0;
    maxExp = ls{k,3};
    minExp = ls{k,3} - length(ls{k,2})+1; 
    poly = ls{k,2};
    liftStep =  ls{k,1};
    
    switch liftStep
       case 'd'
        if(minExp<=0)
          tempLeftExtension = 2*abs(minExp) + parBegin ;
        end
        if(maxExp>0)
           tempRightExtension = 2*maxExp - 2 + parEnd ;
        end
       case 'p' 
        if(minExp<0)
          tempLeftExtension = 2*abs(minExp) - 1 - parBegin;
        end
         if(maxExp>=0)
           tempRightExtension = 2*maxExp + 1 - parEnd ;
        end
        otherwise
            error('Only d or p lifting step are allowed!');
    end

  
    leftExtension = leftExtension + tempLeftExtension;
    rightExtension = rightExtension + tempRightExtension;
    parBegin = mod(mod(leftExtension,2) + mod(beginS,2),2);
    parEnd = mod(mod(rightExtension,2) + mod(endS,2),2);
end



